package Presenter;

import Model.Line;
import Model.Pair;
import Model.Slab;
import Presenter.Comparators.YOrderLineComparatorBegin;
import Presenter.Comparators.YOrderLineComparatorEnd;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

/**
 * Implementierung verschiedener Algorithmen zur Berechnung von Ausgleichsgeraden.
 *
 * @Author: Armin Wolf
 * @Email: a_wolf28@uni-muenster.de
 * @Date: 18.06.2017.
 */
public class InversionCounter {

  private HashMap<Integer, Integer> dictionaryTO;
  private HashMap<Integer, Integer> dictionaryBACK;
  private ArrayList<Integer> substituted;
  private ArrayList<Pair> inversions;
  private List<Line> set;

  //indexieren der Punkte damit die schnittpunkte berechnet werden können
  private HashMap<Line, Integer> secondaryDictionaryTO;
  private HashMap<Integer, Line> secondaryDictionaryBACK;
  private ArrayList<Line> umin;


  public int run(List<Integer> a, List<Integer> b) {

    dictionaryTO = new HashMap<>();
    dictionaryBACK = new HashMap<>();
    substituted = new ArrayList<>();
    inversions = new ArrayList<>();

    ArrayList<Integer> temp = new ArrayList<>();

    temp.addAll(a);

    for (int i = 0; i < a.size(); i++) {
      dictionaryTO.put(a.get(i), i + 1);
      dictionaryBACK.put(i + 1, a.get(i));
    }

    for (int j = 0; j < b.size(); j++) {
      substituted.add(dictionaryTO.get(b.get(j)));
    }

    int ret = countInversions(substituted, 0, substituted.size() - 1, temp);

    getIntersectionAbscissas();

    return ret;
  }

  public int run(List<Line> set, Slab slab) {
    ArrayList<Integer> listA = new ArrayList<>();
    ArrayList<Integer> listB = new ArrayList<>();

    prepareData(set, slab, listA, listB);
    return run(listA, listB);
  }


  private void prepareData(List<Line> set, Slab slab, ArrayList<Integer> listA,
      ArrayList<Integer> listB) {
    secondaryDictionaryTO = new HashMap<>();
    secondaryDictionaryBACK = new HashMap<>();
    this.set = set;
    umin = new ArrayList<>();
    Line tmpLine;

    for (Line p : set) {
      //vertauscht das Point standardmäßig die x lexikografische Ordnung betrachtet
      tmpLine = new Line(p.getM(), p.getM(), slab.getLower() * p.getM() + p.getB(),
          slab.getUpper() * p.getM() + p.getB());
      //wird benötigt um späer die Schnittpunkte ermitteln zu können
      tmpLine.setB(p.getB());
      tmpLine.setM(p.getM());
      umin.add(tmpLine);
    }

    for (int i = 0; i < umin.size(); i++) {
      secondaryDictionaryTO.put(umin.get(i), i);
      secondaryDictionaryBACK.put(i, this.set.get(i));
    }

    Collections.sort(umin, new YOrderLineComparatorBegin());
    for (Line q : umin) {
      listA.add(secondaryDictionaryTO.get(q));
    }

    Collections.sort(umin, new YOrderLineComparatorEnd());
    for (Line q : umin) {
      listB.add(secondaryDictionaryTO.get(q));
    }

  }


  /**
   * Angepasster Merge-Sort Algorithmus.
   * Die Funktion bekommt neben den standard Parametern zusätzlich eine Liste mit Elementen
   * die als Groundtruth dienen.
   *
   * @param a Eingabefeld mit den Elementen die überprüft werden sollen.
   * @param start Startpunkt des Eingabefeldes.
   * @param end Endpunkt des Eingabefeldes.
   * @param aux Groundtruth Ordnung um die Anzahl der Inversionen zu bestimmen.
   * @return Anzahl der inversionen zwischen a und aux.
   */
  public int countInversions(List<Integer> a, int start, int end, List<Integer> aux) {
    if (start >= end) {
      return 0;
    }
    int invCount = 0;
    int mid = start + (end - start) / 2;
    int invCountLeft = countInversions(a, start, mid, aux); // divide and conquer
    int invCountRight = countInversions(a, mid + 1, end, aux); // divide and conquer
    invCount += (invCountLeft + invCountRight);
    for (int i = start; i <= end; i++) {
      aux.set(i, a.get(i));
    }
    int left = start;
    int right = mid + 1;
    int index = start;
    while (left <= mid && right <= end) {
      if (aux.get(left) < aux.get(right)) {
        a.set(index++, aux.get(left++));
      } else {

        for (int i = left; i <= mid; i++) {
          // System.out.println(aux.get(i)+" -- "+ aux.get(right));
          inversions.add(new Pair(aux.get(i), aux.get(right)));
        }
        a.set(index++, aux.get(right++));
        invCount += mid - left + 1; // number of inversions for aux[right]
      }
    }
    while (left <= mid) {
      a.set(index++, aux.get(left++));
    }
    // no need to copy over remaining aux[right++] because they are already inside a
    return invCount;
  }


  public HashMap<Line, ArrayList<Line>> getIntersectionAbscissas() {
    ArrayList<Pair> result = new ArrayList<>();
    HashMap<Line, ArrayList<Line>> ret = new HashMap<>();

    for (int i = 0; i < inversions.size(); i++) {
      result.add(new Pair(dictionaryBACK.get(inversions.get(i).getP1()),
          dictionaryBACK.get(inversions.get(i).getP2())));
    }
    ArrayList<Line> linePairs;

    for (Pair p : result) {
      Line l1 = secondaryDictionaryBACK.get(p.getP1());
      Line l2 = secondaryDictionaryBACK.get(p.getP2());
      if (ret.get(l2) == null){
        linePairs = new ArrayList<>();
      } else {
        linePairs = ret.get(l2);
      }
      linePairs.add(l1);
      ret.put(l2, linePairs);

      //Symetrie
      if (ret.get(l1) == null){
        linePairs = new ArrayList<>();
      } else {
        linePairs = ret.get(l1);
      }
      linePairs.add(l2);
      ret.put(l1, linePairs);
    }

//    System.out.println("----------------------------------------------------------");
//    for (Line outerLine : ret.keySet()){
//      System.out.println("Linie: "+outerLine);
//      for (Line innerLine : ret.get(outerLine)){
//        System.out.println("\t\t -> "+innerLine);
//      }
//    }
//    System.out.println("----------------------------------------------------------");
    return ret;
  }

}
